#include "window.h"
#include "ui_window.h"

#include "shape.h"
#include "colorbutton.h"
#include "shapebutton.h"

#include <QApplication>

#include <QDrag>
#include <QMimeData>
#include <QIcon>
#include <QGraphicsSceneDragDropEvent>
#include <QGraphicsRectItem>

#include <cassert>

/* static */

MainWindow * MainWindow::instance = NULL;

void refreshTree ()
{
    if (MainWindow::instance != NULL)
        MainWindow::instance->updateTree ();
}

/* scene selection changed */

void MainWindow::scene_selection_changed()
{
    QList <QGraphicsItem *> sel = scene->selectedItems ();
    if (sel.count() == 1)
    {
        QGraphicsItem * item = sel [0];
        current_item = item;
    }
    else
    {
        current_item = NULL;
    }
}

/* tree */

void MainWindow::updateBranch (QTreeWidgetItem * branch, QGraphicsItem * item)
{
    TreeItem * node = new TreeItem;
    node->shape = item;
    QString s = "item";
    if (dynamic_cast <Rectangle*> (item) != NULL)
        s = "rectangle";
    else if (dynamic_cast <Ellipse*> (item) != NULL)
        s = "ellipse";
    node->setText (0, s);
    QAbstractGraphicsShapeItem *shape = dynamic_cast <QAbstractGraphicsShapeItem *> (item);
    if (shape != NULL)
    {
       // node->setForeground (0, shape->pen().color());
       // node->setBackground (0, shape->brush().color());
       node->setForeground (0, shape->brush().color());
    }
    branch->addChild (node);

    for (QGraphicsItem * t : item->childItems())
        updateBranch (node, t);
}

void MainWindow::updateTree()
{
    ui->tree->clear ();
    QTreeWidgetItem * branch = ui->tree->invisibleRootItem();
    for (QGraphicsItem * item : scene->items ())
    {
        if (item->parentItem() == NULL)
            updateBranch (branch, item);
    }
    ui->tree->expandAll();
}

void MainWindow::on_tree_itemDoubleClicked (QTreeWidgetItem *item, int column)
{
    TreeItem * t = dynamic_cast <TreeItem *> (item);
    if (t != NULL)
    {
        for (QGraphicsItem * s : scene->selectedItems())
            s->setSelected (false);
        t->shape->setSelected (true);
        updateTable (t->shape);
    }
}

void MainWindow::addLine (QString name, QVariant value)
{
   int n = ui->prop->rowCount();
   ui->prop->setRowCount (n+1);

   QTableWidgetItem * t = new QTableWidgetItem;
   t->setText (name);
   ui->prop->setItem (n, 0, t);

   t = new QTableWidgetItem;
   t->setText (value.toString());
   ui->prop->setItem (n, 1, t);

   QVariant::Type type = value.type();
   if (type == QVariant::Pen)
   {
       QColor color = value.value <QPen> ().color ();
       t->setData(Qt::EditRole, color);
       t->setData(Qt::DisplayRole, color);
       t->setData(Qt::DecorationRole, color);
   }
   if (type == QVariant::Brush)
   {
       QColor color = value.value <QBrush> ().color ();
       t->setData(Qt::EditRole, color);
       t->setData(Qt::DisplayRole, color);
       t->setData(Qt::DecorationRole, color);
   }

}

void MainWindow::updateTable (QGraphicsItem * item)
{
    ui->prop->setRowCount(0);
    ui->prop->setColumnCount(2);
    ui->prop->setHorizontalHeaderLabels (QStringList() << "name" << "value");

    addLine ("x", item->x());
    addLine ("y", item->y());

    QAbstractGraphicsShapeItem *shape = dynamic_cast <QAbstractGraphicsShapeItem *> (item);
    if (shape != NULL)
    {
       addLine ("pen", shape->pen());
       addLine ("brush", shape->brush());
    }

}

/* MainWindow */

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow (parent),
    ui(new Ui::MainWindow),
    scene (NULL),
    current_item (NULL)
{
    ui->setupUi(this);
    instance = this;

    /* setup tools */

    ui->mainToolBar->addWidget (new ShapeButton ("ellipse"));
    ui->mainToolBar->addWidget (new ShapeButton ("rectangle"));

    ui->mainToolBar->addSeparator ();

    /* setup colors */

    QList <QString> names;
    names << "red" << "blue" << "green" << "yellow" << "orange";

    for (auto name : names)
    {
       ColorButton * b = new ColorButton (name);
       ui->mainToolBar->addWidget (b);
    }

    /* setup scene */

    scene = new QGraphicsScene;
    ui->graphicsView->setScene (scene);

    QObject::connect (scene, SIGNAL (selectionChanged()),
                      this, SLOT (scene_selection_changed()));

    scene->setSceneRect (0, 0, 1280, 1024);

    Plane * p = new Plane;
    scene->addItem (p);

    Rectangle * r = new Rectangle;
    r->setParentItem (p);

    Ellipse * e = new Ellipse;
    e->setParentItem (p);

    /* setup window */

     ui->input->setVisible (false);

     ui->hsplitter->setStretchFactor (0, 1);
     ui->hsplitter->setStretchFactor (1, 4);
     ui->hsplitter->setStretchFactor (2, 1);

     ui->vsplitter->setStretchFactor (0, 4);
     ui->vsplitter->setStretchFactor (1, 1);
     ui->vsplitter->setStretchFactor (2, 1);

     updateTree ();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_actionQuit_triggered()
{
   this->close ();
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    MainWindow w;
    w.show();
    return a.exec();
}

